"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = exports.app = void 0;
const lodash_1 = __importDefault(require("lodash"));
const http_auth_1 = __importDefault(require("http-auth"));
const http_auth_connect_1 = __importDefault(require("http-auth-connect"));
const body_parser_1 = __importDefault(require("body-parser"));
const bcrypt_1 = __importDefault(require("bcrypt"));
const compression_1 = __importDefault(require("compression"));
const express_1 = __importDefault(require("express"));
const graceful_fs_1 = require("graceful-fs");
const config_1 = require("./config");
const types_1 = require("./types");
const logger_1 = __importDefault(require("./logger"));
exports.app = (0, express_1.default)();
/**
 * Used to have some password comparison in case the user does not exist to avoid timing attacks.
 */
const INVALID_PASSWORD_HASH = "$2b$05$JebmV1q/ySuxa89GoJYlc.6SEnj1OZYBOfTf.TYAehcC5HLeJiWPi";
/**
 * Trying to implement a timing safe string compare.
 *
 * TODO: Write tests for timing.
 */
function timingSafeEqual(a, b) {
    const lenA = a.length;
    const lenB = b.length;
    // Greater than 0 for differing strings.
    let different = Math.abs(lenA - lenB);
    // Make sure b is always the same length as a. Use slice to try avoiding optimizations.
    b = different === 0 ? b.slice() : a.slice();
    for (let i = 0; i < lenA; i += 1) {
        different += Math.abs(a.charCodeAt(i) - b.charCodeAt(i));
    }
    return different === 0;
}
async function isValidLogin(username, password) {
    if (!config_1.config.server.internal.active) {
        return false;
    }
    let passwordHash = undefined;
    // Iterate over all users every time to reduce risk of timing attacks.
    for (const userConfig of config_1.config.server.internal.users) {
        if (timingSafeEqual(username, userConfig.username)) {
            passwordHash = userConfig.passwordHash;
        }
    }
    // Always compare some password even if the user does not exist to reduce risk of timing attacks.
    const isValidPassword = await bcrypt_1.default.compare(password, passwordHash || INVALID_PASSWORD_HASH);
    // Make sure password is only considered valid is user exists and therefor passwordHash is not undefined.
    return (0, types_1.isString)(passwordHash) && isValidPassword;
}
function init() {
    const router = express_1.default.Router();
    // urls beneath /internal are protected
    const internalAuth = http_auth_1.default.basic({
        realm: 'Knotenformular - Intern'
    }, function (username, password, callback) {
        isValidLogin(username, password)
            .then(result => callback(result))
            .catch(err => {
            logger_1.default.tag('login').error(err);
        });
    });
    router.use('/internal', (0, http_auth_connect_1.default)(internalAuth));
    router.use(body_parser_1.default.json());
    router.use(body_parser_1.default.urlencoded({ extended: true }));
    const adminDir = __dirname + '/../admin';
    const clientDir = __dirname + '/../client';
    const templateDir = __dirname + '/templates';
    const jsTemplateFiles = [
        '/config.js'
    ];
    function usePromise(f) {
        router.use((req, res, next) => {
            f(req, res).then(next).catch(next);
        });
    }
    router.use((0, compression_1.default)());
    async function serveTemplate(mimeType, req, res) {
        const body = await graceful_fs_1.promises.readFile(templateDir + '/' + req.path + '.template', 'utf8');
        res.writeHead(200, { 'Content-Type': mimeType });
        res.end(lodash_1.default.template(body)({ config: config_1.config.client }));
    }
    usePromise(async (req, res) => {
        if (jsTemplateFiles.indexOf(req.path) >= 0) {
            await serveTemplate('application/javascript', req, res);
        }
    });
    router.use('/internal/admin', express_1.default.static(adminDir + '/'));
    router.use('/', express_1.default.static(clientDir + '/'));
    exports.app.use(config_1.config.server.rootPath, router);
}
exports.init = init;
//# sourceMappingURL=app.js.map